format binary as ""
use32
org	0x0
db	'MENUET01'
dd	1, START, I_END, mem, stacktop, 0, sys_path

; Common libs
include 'proc32.inc'
include 'macros.inc'
include 'dll.inc'
include 'box_lib.mac'
include 'load_lib.mac'
	@use_library_mem mem.Alloc,mem.Free,mem.ReAlloc,dll.Load
; XMPP libs
include 'net.asm'
include 'common.asm'
include 'xmpp_xml.asm'
include 'xmpp_const.asm'
include 'xmpp_stream.asm'
include 'xmpp_attr.asm'
include 'xmpp_msgbox.asm'
include 'xmpp_login.asm'
include 'xmpp_main.asm'
; Custom libs
include 'rxl_node.inc'
include 'rxl_attr.inc'
include 'rxl_read.inc'
include 'rxl_write.inc'


; Socket buffer size
SNDBUFFER		= 0x1000
RCVBUFFER		= 0x1000

; Network consts
STATUS_DISCONNECTED	= 0
STATUS_RESOLVING	= 1
STATUS_CONNECTING	= 2
STATUS_CONNECTED	= 3
STATUS_LOGIN		= 4
STATUS_ONLINE		= 5



;<<<<<<<<<<<<<<<<<<<
;
;       CODE BLOCK
;
;<<<<<<<<<<<<<<<<<<<
align 16
START:
	; Heap init
	mcall	68,11

	; Load libs
	stdcall dll.Load, @IMPORT
	cmp	eax,-1
	jz	close_progr
	; Event mask
	mcall	40, EVM_REDRAW + EVM_KEY + EVM_BUTTON + EVM_MOUSE + EVM_IPC + EVM_STACK
	test	eax,eax
	jz	close_progr
	; Get system colors
	call	get_sys_colors
	; Prepare socket buffers
	stdcall mem.Alloc, DWORD RCVBUFFER
	mov	[rcv_buffer], eax
	stdcall mem.Alloc, DWORD SNDBUFFER
	mov	[snd_buffer], eax
	; Draw window
	call	login_win_draw

wait_action:
	cmp	[status],STATUS_LOGIN
	jle	.wait_login
	jmp	.wait_online

     .wait_login:
	call	wait_login
	jmp	.final

     .wait_online:
	call	wait_main
	jmp	.final

     .final:
	jmp	wait_action


get_sys_colors:
	; Push all regs to stack saving them
	pusha
	; fn 48 - sys colors
	mov	eax, 48
	; fn 48.ebx - get
	mov	ebx, 3
	; fn 48.3.ecx - pointer to var sys_colours
	mov	ecx, sc
	; fn 48.3.edx - return 40 bytes
	mov	edx, 10*4
	mcall
	; Pop all regs
	popa
	; Return
	ret

free_alloc_memory:
	push	esi
	call	[freeaddrinfo]
	ret


close_progr:
	mcall	-1


;<<<<<<<<<<<<<<<<<<<
;
;       DATA BLOCK
;
;<<<<<<<<<<<<<<<<<<<
x_stream	xmpp_stream

; Sys colours table
sc system_colors

;<<<<<<<<<<<<<<<<<<<<<<<<
; NETWORK
;<<<<<<<<<<<<<<<<<<<<<<<<
sockaddr1:
	dw	AF_INET4
	.port	dw 0
	.ip	dd 0
	rb	10		; Address buffer

ip_ptr		dd ?
socketnum	dd ?
status	    dd STATUS_DISCONNECTED

;<<<<<<<<<<<<<<<<<<<<<<<<<
; LIBRARY
;<<<<<<<<<<<<<<<<<<<<<<<<<
align 4
@IMPORT:

library network, 'network.obj', \
	box_lib, 'box_lib.obj', \
	lib_img, 'libimg.obj'
import	network, \
	getaddrinfo	     , 'getaddrinfo',\
	freeaddrinfo	     , 'freeaddrinfo',\
	inet_ntoa	     , 'inet_ntoa'
import	box_lib, \
	boxlib_init	     , 'lib_init',\
	edit_box_draw	     , 'edit_box',\
	edit_box_key	     , 'edit_box_key',\
	edit_box_mouse	     , 'edit_box_mouse',\
	version_ed	     , 'version_ed',\
	scrollbar_ver_draw   , 'scrollbar_v_draw',\
	scrollbar_ver_mouse  , 'scrollbar_v_mouse',\
	scrollbar_hor_draw   , 'scrollbar_h_draw',\
	scrollbar_hor_mouse  , 'scrollbar_h_mouse',\
	tl_data_init	     , 'tl_data_init',\
	tl_data_clear	     , 'tl_data_clear',\
	tl_info_clear	     , 'tl_info_clear',\
	tl_key		     , 'tl_key',\
	tl_mouse	     , 'tl_mouse',\
	tl_draw 	     , 'tl_draw',\
	tl_info_undo	     , 'tl_info_undo',\
	tl_info_redo	     , 'tl_info_redo',\
	tl_node_add	     , 'tl_node_add',\
	tl_node_set_data     , 'tl_node_set_data',\
	tl_node_get_data     , 'tl_node_get_data',\
	tl_node_delete	     , 'tl_node_delete',\
	tl_cur_beg	     , 'tl_cur_beg',\
	tl_cur_next	     , 'tl_cur_next',\
	tl_cur_prev	     , 'tl_cur_perv',\
	tl_node_close_open   , 'tl_node_close_open',\
	tl_node_lev_inc      , 'tl_node_lev_inc',\
	tl_node_lev_dec      , 'tl_node_lev_dec',\
	tl_node_move_up      , 'tl_node_move_up',\
	tl_node_move_down    , 'tl_node_move_down',\
	ted_init	     , 'ted_init',\
	ted_version	     , 'ted_version',\
	ted_draw	     , 'ted_draw',\
	ted_clear	     , 'ted_clear',\
	ted_init_scroll_bars , 'ted_init_scroll_bars',\
	ted_init_syntax_file , 'ted_init_syntax_file',\
	ted_is_select	     , 'ted_is_select',\
	ted_key 	     , 'ted_key',\
	ted_mouse	     , 'ted_mouse',\
	ted_text_add	     , 'ted_text_add'
import	lib_img,\
	imglib_init	     , 'lib_init',\
	img_as_img	     , 'img_as_img',\
	img_info	     , 'img_info',\
	img_from_file	     , 'img_from_file',\
	img_to_file	     , 'img_to_file',\
	img_from_rgb	     , 'img_from_rbg',\
	img_to_rgb	     , 'img_to_rgb',\
	img_to_rgb2	     , 'img_to_rgb2',\
	img_decode	     , 'img_decode',\
	img_encode	     , 'img_encode',\
	img_create	     , 'img_create',\
	img_destroy	     , 'img_destroy',\
	img_destroy_layer    , 'img_destroy_layer',\
	img_count	     , 'img_count',\
	img_lock_bits	     , 'img_lock_bits',\
	img_unlock_bits      , 'img_unlock_bits',\
	img_flip	     , 'img_flip',\
	img_flip_layer	     , 'img_flip_layer',\
	img_rotate	     , 'img_rotate',\
	img_rotate_layer     , 'img_rotate_layer',\
	img_draw	     , 'img_draw'


RtnAdrs 	dd 0
BasePtr 	dd 0
; End of program
I_END:
; Additional buffer
		rb 1024
; Recieve buffer
msg_len 	dd 0
snd_buffer	dd 0
rcv_buffer	dd 0
tmp_buffer	rb 1024
sys_path	rb 4096
file_name	rb 4096
library_path	rb 4096
	rb	1024
stacktop:
mem:
