; Clean given buffer with zeros
proc cleanbuf lpBuf:DWORD,length
	push	edi
	mov	ecx,[length]
	mov	edi,[lpBuf]
@@:
	mov	[edi],byte 0
	inc	edi
	dec	ecx
	test	ecx,ecx
	jnz	@b
	pop	edi
	ret
endp

; Strlen by Heavyiron
proc strlen lpStr
	mov	ecx,[lpStr]
	dec	ecx
@@:
	inc	ecx
	cmp	byte [ecx], 0
	jne	@b
	sub	ecx, [lpStr]
	ret
endp

; Strcpy
; Copies 0-terminated string of unknown length and returns the [last byte+1] address as edi
; Returns
;       edi - points to the next char after copied string
proc strcpy lpSrc,lpDst:DWORD
	push	esi
	stdcall   strlen,[lpSrc]
	mov	esi, [lpSrc]
	mov	edi, [lpDst]
	cld
	rep	movsb
	pop	esi
	ret
endp

; Strcpy_l
; Copies string of a given length and returns the [last byte+1] address as edi
; Returns
;       edi - points to the next char after copied string
proc strcpy_l lpSrc,lpDst,length
	push	esi
	mov	ecx, [length]
	mov	esi, [lpSrc]
	mov	edi, [lpDst]
	cld
	rep	movsb
	pop	esi
	ret
endp


; Strstr
; Finds substring in string buffer
; Returns:
;       eax = position of the first char if found, else 0
; From Asterix, http://wasm.ru, topic id 27362
proc strstr lpBuf:DWORD,lenBuf:DWORD,lpStr:DWORD,lenStr:DWORD
	push	ebx ecx edx esi edi
	mov	edi,[lpStr]
	mov	esi,[lpBuf]
	mov	ecx,[lenStr]
	mov	ebx,ecx
	mov	edx,esi
	add	edx,[lenBuf]
	sub	edx,[lenStr]
	inc	edx
     .loop:
	cmp	esi,edx
	je	.not_found
	test	ecx,ecx
	jz	.found
	mov	al, byte [edi]
	test	al,al
	jz	.h_found
	cmp	byte [esi], al
	je	.parse
	sub	edi,ebx
	add	edi,ecx
	sub	esi,ebx
	add	esi,ecx
	inc	esi
	mov	ecx,ebx
	jmp	.loop
     .parse:
	inc	edi
	inc	esi
	dec	ecx
	jmp	.loop
     .h_found:
	inc	edi
	inc	esi
	dec	ecx
	jmp	.loop
     .not_found:
	xor	eax,eax
	jmp	@f
     .found:
	mov	eax,esi
	sub	eax,ebx
	sub	eax,[lpBuf]
     @@:
	pop	edi esi edx ecx ebx
	ret
endp

proc clrbuf dBuf,len
	mov	edi, [dBuf]
	mov	ecx, len
@@:
	mov	byte[edi],0
	inc	edi
	dec	ecx
	jnz	@b
	ret
endp

; Base64 encoder
; Encodes given buffer into base64 stream
; Returns also output size in eax
; Written by manhunter, http://www.manhunter.ru
base64_table db 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/',0

proc base64_encode, .source,.src_size,.out_buf
	push	ebx ecx edx esi edi
	mov	esi,[.source]
	mov	edi,[.out_buf]
	mov	ecx,[.src_size]

	or	ecx,ecx
	jz	.r3

     .encode_loop:
	lodsd
	mov	edx,eax
	cmp	ecx,3
	jae	.remainder_ok
	and	edx,0xffff
	cmp	ecx,2
	jae	.remainder_ok
	and	edx,0xff
     .remainder_ok:
	bswap	edx

	mov	eax,edx
	shr	eax,26
	and	eax,00111111b
	mov	al,[base64_table+eax]
	stosb

	mov	eax,edx
	shr	eax,20
	and	eax,00111111b
	mov	al,[base64_table+eax]
	stosb
	dec	ecx
	jz	.r2

	mov	eax,edx
	shr	eax,14
	and	eax,00111111b
	mov	al,[base64_table+eax]
	stosb
	dec	ecx
	jz	.r1

	mov	eax,edx
	shr	eax,8
	and	eax,00111111b
	mov	al,[base64_table+eax]
	stosb

	dec	esi
	dec	ecx
	jnz	.encode_loop
	jmp	.r3
     .r2:
	mov	al,'='
	stosb
     .r1:
	mov	al,'='
	stosb
     .r3:
	xor	eax,eax
	stosb
	pop	edi esi edx ecx ebx
	ret
endp


; Base64 decoder
; Taken from manhunter.ru
decode_table	rb 100h
proc base64_decode .source,.src_size,.out_buf
	pusha
	mov	ecx,[.src_size]
	mov	esi,[.source]
	mov	edi,[.out_buf]
	xor	eax,eax
	mov	ebx,63

      .make_table:
	mov	al,[base64_table+ebx]
	mov	[decode_table+eax],bl
	dec	ebx
	jns	.make_table
	mov	[decode_table+'='],0
	xor	ebx,ebx
      .load_dword:
	mov	edx,[esi]
	bswap	edx
	add	esi,4

	mov	al,dl
	mov	al,byte[decode_table+eax]
	shrd	ebx,eax,6
	shr	edx,8

	mov	al,dl
	mov	al,byte[decode_table+eax]
	shrd	ebx,eax,6
	shr	edx,8

	mov	al,dl
	mov	al,byte[decode_table+eax]
	shrd	ebx,eax,6
	shr	edx,8

	mov	al,dl
	mov	al,byte[decode_table+eax]
	shrd	ebx,eax,6

	bswap	ebx
	mov	[edi],ebx
	add	edi,3
	sub	ecx,4
	js	.exit_loop
	jnz	.load_dword

      .exit_loop:
	mov	byte[edi],0
	inc	edi
	mov	eax,[.out_buf]
	sub	edi,eax
	mov	eax,edi
	dec	eax
	popa
endp


