; Finds XML tag opening position for a given lpTag in a given lpBuf
; Returns:
;       edi - pointer set just before tag opening (before <)
;       If tag was not found - returns 0
proc findXmlOpen lpTag,lpBuf,lenBuf
	push	eax ebx ecx edx esi
	; Get tag length
	stdcall strlen,[lpTag]
	mov	ebx,ecx
	; Set initial pointer
	mov	edi,[lpBuf]
	; Find next <
     @@:
	mov	ax,ds
	mov	es,ax
	cld
	mov	al,'<'
	mov	ecx,[lenBuf]
	add	ecx,[rcv_buffer]
	sub	ecx,edi
	repne	scasb
	je	.opening
	jmp	.not_found
	; If < found - compare following chars to tag
     .opening:
	mov	ecx,ebx
	mov	esi,[lpTag] ; '<' position already in di
	repe	cmpsb
	jne	@b
	; If tag is good - return ebx+1 steps back
	pop	ecx
	sub	edi,ebx
	dec	edi
	jmp	.final
     .not_found:
	mov	edi,0
     .final:
	pop	esi edx ecx ebx eax
	ret
endp


proc createXmlOpen name,type,buffer
; Usage:
;       Name - xml node name
;       Type - node type, e.g. <name:type args>
;       Buffer - xml string buffer
;       Parameters are added as additional args
;       in order label,data,label,data...
;       Last par should be 0
;       Invoke using ccall and REMEMBER to correct stack by 3 doubles after
; Example:
;       ccall   createXmlOpen,name,type,buffer,par1,val1,0
;       sub     esp,0xC
; Output:
;       In buffer - xml string
;       In edi - last byte
	mov	edi,[buffer]
; Add opening
	mov	byte [edi],'<'
	inc	edi
; Write node name
	stdcall strcpy,[name],edi
; Check if node type is present
	stdcall strlen,[type]
	cmp	ecx, 0
	jz	@f
; If type is present
	mov	byte [edi], ':'
	inc	edi
	stdcall strcpy,[type],edi
@@:
; Parameters are 5 doubles away, initial esi set to init stack + 4 doubles
	mov	esi,ebp
	add	esi,16
@@:
; Add parameter name
	add	esi,4
	cmp	byte [esi],0
	jz	@f
	mov	byte [edi], ' '
	inc	edi
	stdcall strcpy,[esi],edi
; Add parameter value
	mov	byte [edi], '='
	inc	edi
	mov	byte [edi], 39
	inc	edi
	add	esi,4
	stdcall strcpy,[esi],edi
	mov	byte [edi], 39
	inc	edi
	jmp	@b
@@:
; Add closing
	mov	byte [edi], '>'
	inc	edi
	mov	byte [edi], 0
	ret
endp

proc createXmlShort name,type,buffer
; Usage:
;       Name - xml node name
;       Type - node type, e.g. <name:type args>
;       Buffer - xml string buffer
;       Parameters are added as additional args
;       in order label,data,label,data...
;       Last par should be 0
;       Invoke using ccall and REMEMBER to correct stack by 3 doubles after
; Example:
;       ccall   createXmlOpen,name,type,buffer,par1,val1,0
;       sub     esp,0xC
; Output:
;       In buffer - xml string
;       In eax - last byte
	mov	edi,[buffer]
; Add opening
	mov	byte [edi],'<'
	inc	edi
; Write node name
	stdcall strcpy,[name],edi
; Check if node type is present
	stdcall strlen,[type]
	cmp	ecx, 0
	jz	@f
; If type is present
	mov	byte [edi], ':'
	inc	edi
	stdcall strcpy,[type],edi
@@:
; Parameters are 5 doubles away, initial esi set to init stack + 4 doubles
	mov	esi,ebp
	add	esi,16
@@:
; Add parameter name
	add	esi,4
	cmp	byte [esi],0
	jz	@f
	mov	byte [edi], ' '
	inc	edi
	stdcall strcpy,[esi],edi
; Add parameter value
	mov	byte [edi], '='
	inc	edi
	mov	byte [edi], 39
	inc	edi
	add	esi,4
	stdcall strcpy,[esi],edi
	mov	byte [edi], 39
	inc	edi
	jmp	@b
@@:
; Add closing
	mov	byte [edi], '/'
	inc	edi
	mov	byte [edi], '>'
	inc	edi
	mov	byte [edi], 0
	ret
endp


proc createXmlClose name,type,buffer
; Usage:
;       Name - xml node name
;       Type - node type, e.g. <name:type args>
;       Buffer - xml string buffer
	mov	edi,[buffer]
; Add opening
	mov	byte [edi],'<'
	inc	edi
	mov	byte [edi],'/'
	inc	edi
; Write node name
	stdcall strcpy,[name],edi
; Check if node type is present
	stdcall strlen,[type]
	cmp	ecx, 0
	jz	@f
; If type is present
	mov	byte [edi], ':'
	inc	edi
	stdcall strcpy,[type],edi
@@:
; Add closing
	mov	byte [edi],'>'
	inc	edi
	mov	byte [edi],0
	ret
endp
